/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib;

/**
 * History:
 * [KS] 2006-08-01 : Konstruktoren und unsubscribeSender() public gemacht
 *
 * $Log: SenderSubscribtion.java,v $
 * Revision 1.2  2007/07/25 13:02:05  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.9  2007/01/16 10:03:18  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.8  2006/11/13 09:24:10  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.7  2006/10/09 05:49:03  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.6  2006/08/17 07:34:00  Strey
 * Konstruktoren und unsubscribeSender() public gemacht
 *
 * Revision 1.5  2006/06/30 09:42:42  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2006/06/30 07:47:54  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2006/06/02 14:23:55  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2006/06/02 13:10:54  Strey
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2006/06/02 13:04:09  Trier
 * initial
 *
 *
 */
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.dataIdentificationSettings.DataIdentification;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * <p>
 * Klasse zum Versenden von Daten.
 * </p>
 * <p>
 * Die Methode {@link #sendData(ResultData rs) sendData(ResultData)} dient dem
 * Versenden von Datens&auml;tzen. In dieser Methode erfolgt auch die
 * Sendeanmeldung.
 * </p>
 * <p>
 * Es wird erst versucht sich als Quelle anzumelden. Ist die nicht erfolgreich
 * (andere Quelle oder Senke vorhanden), wird versucht sich als Sender
 * anzumelden.
 * </p>
 * <p>
 * Kann der Datensatz nicht versendet werden, wird eine Exception geworfen.
 * </p>
 * <p>
 * Mit der Methode {@link #unsubscribeSender() } kann die Sendeanmeldung
 * zur&uuml;ckgenommen werden.
 * </p>
 * <p>
 * Wird die {@link #sendData(ResultData rs) sendData(ResultData) } danach
 * aufgerufen wird die Sendeanmeldung wiederhergestellt.
 * </p>
 * <p>
 * Es wird eine Timeout-Zeit auf eine positive Sendesteuerung gewartet. Ist
 * diese Zeit &uuml;berschritten, ist wahrscheinlich weder Senke noch Empfänger
 * vorhanden.
 * </p>
 *
 * @author Ralf Trier
 * @version 1.0
 */
public class SenderSubscribtion implements ClientSenderInterface {

	private static final Debug LOGGER = Debug.getLogger();

	enum State {
		NOT_INITIALIZED, SUBCRIBED_SOURCE, SUBCRIBED_SENDER, SUBCRIBTION_ERROR, SUBCRIBTION_ERROR_NOT_A_VALID_SUBSCRIPTION
	};

	private boolean _IsTimeOut = false;
	private boolean _IsSendingAllowed = false;

	private State _State = State.NOT_INITIALIZED;

	private final ClientDavInterface _DavConn;
	private final DataIdentification _DataIdentification;
	private long _LastUse = System.currentTimeMillis();

	/**
	 * Ist _IgnoreStopSending==true, dann wird auch die TimeoutZeit auf eine
	 * positive Sendesteuerung gewartet, wenn ein STOP_SENDING ausgel&ouml;st
	 * wurde. Normalerweise wird davon ausgegangen, dass wenn STOP_SENDING
	 * ausgel&ouml;st wurde, dass weder Quelle noch Senke vorhanden ist.
	 */
	protected boolean _IgnoreStopSending = false;

	/**
	 * Zeit in msec, die gewartet wird, dass das Senden erlaubt wird.
	 */
	private int _TimeOut = 5000;

	/**
	 * erzeugt eine SenderSubscribtion.
	 *
	 * @param davConn
	 *            ClientDavInterface
	 * @param so
	 *            SystemObject
	 * @param dataDescription
	 *            DataDescription
	 */
	public SenderSubscribtion(final ClientDavInterface davConn, final SystemObject so,
			final DataDescription dataDescription) {
		_DavConn = davConn;
		_DataIdentification = new DataIdentification(so, dataDescription);
	}

	/**
	 * erzeugt eine SenderSubscribtion.
	 *
	 * @param davConn
	 *            ClientDavInterface
	 * @param so
	 *            SystemObject
	 * @param dataDescription
	 *            DataDescription
	 * @param timeOut
	 *            Zeit in msec, die auf positive Sendesteuerung gewartet wird
	 */
	public SenderSubscribtion(final ClientDavInterface davConn, final SystemObject so,
			final DataDescription dataDescription, final int timeOut, final boolean ignoreStopSending) {
		_DavConn = davConn;
		_DataIdentification = new DataIdentification(so, dataDescription);
		_TimeOut = timeOut;
		_IgnoreStopSending = ignoreStopSending;
	}

	/**
	 * erzeugt eine SenderSubscribtion.
	 *
	 * @param davConn
	 *            ClientDavInterface
	 * @param dataIdentification
	 *            DataIdentification
	 */
	public SenderSubscribtion(final ClientDavInterface davConn, final DataIdentification dataIdentification) {
		_DavConn = davConn;
		_DataIdentification = dataIdentification;
	}

	/**
	 * erzeugt eine SenderSubscribtion.
	 *
	 * @param davConn
	 *            ClientDavInterface
	 * @param dataIdentification
	 *            DataIdentification
	 * @param timeout
	 *            Zeit in msec, die auf positive Sendesteuerung gewartet wird
	 * @param ignoreStopSending
	 *            boolean
	 */
	public SenderSubscribtion(final ClientDavInterface davConn, final DataIdentification dataIdentification,
			final int timeout, final boolean ignoreStopSending) {
		_DavConn = davConn;
		_DataIdentification = dataIdentification;
		_TimeOut = timeout;
		_IgnoreStopSending = ignoreStopSending;
	}

	@Override
	public synchronized void dataRequest(final SystemObject object, final DataDescription dataDescription,
			final byte state) {

		if ((_DataIdentification.getDataDescription().getAttributeGroup().equals(dataDescription.getAttributeGroup()))
				&& (_DataIdentification.getDataDescription().getAspect().equals(dataDescription.getAspect()))
				&& (_DataIdentification.getObject().equals(object))) {
			// System.err.println("_IgnoreStopSending/_IsTimeOut:
			// "+_IgnoreStopSending+"/"+_IsTimeOut+" "+this);
			switch (state) {
			case ClientSenderInterface.START_SENDING: {
				_IsSendingAllowed = true;
				_IsTimeOut = false;
				SenderSubscribtion.LOGGER.fine("DataRequest => ClientSenderInterface.START_SENDING", this);
				notifyAll();
				break;
			}
			case ClientSenderInterface.STOP_SENDING: {
				_IsSendingAllowed = false;
				this.unsubscribeSender();
				_State = State.SUBCRIBTION_ERROR;
				SenderSubscribtion.LOGGER.fine("DataRequest => ClientSenderInterface.STOP_SENDING", this);
				if (_IgnoreStopSending) {
					System.err.println("DataRequest01 => ClientSenderInterface.STOP_SENDING " + this);
				} else {
					System.err.println("DataRequest02 => ClientSenderInterface.STOP_SENDING " + this);
					_IsTimeOut = false;
					notifyAll();
				}
				break;
			}
			case ClientSenderInterface.STOP_SENDING_NO_RIGHTS: {
				_IsSendingAllowed = false;
				this.unsubscribeSender();
				_State = State.SUBCRIBTION_ERROR;
				System.err.println("DataRequest03 => ClientSenderInterface.STOP_SENDING_NO_RIGHTS  " + this);
				SenderSubscribtion.LOGGER.fine("DataRequest => ClientSenderInterface.STOP_SENDING_NO_RIGHTS", this);
				_IsTimeOut = false;
				notifyAll();
				break;
			}
			case ClientSenderInterface.STOP_SENDING_NOT_A_VALID_SUBSCRIPTION: {
				_IsSendingAllowed = false;
				this.unsubscribeSender();
				_State = State.SUBCRIBTION_ERROR_NOT_A_VALID_SUBSCRIPTION;
				System.err.println(
						"DataRequest04 => ClientSenderInterface.STOP_SENDING_NOT_A_VALID_SUBSCRIPTION " + this);
				;
				SenderSubscribtion.LOGGER
				.fine("DataRequest => ClientSenderInterface.STOP_SENDING_NOT_A_VALID_SUBSCRIPTION ", this);
				_IsTimeOut = false;
				notifyAll();
				break;
			}
			}
		}
		SenderSubscribtion.LOGGER.fine("nach dataRequest", this);
	}

	/**
	 * gibt true zur&uuml;ck, um zu signalisieren, dass die Sendesteuerung
	 * unterst&uuml;tzt wird.
	 *
	 * @param object
	 *            SystemObject
	 * @param dataDescription
	 *            DataDescription
	 * @return boolean
	 */
	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		return true;
	}

	/**
	 * versendet den Datensatz.
	 *
	 * <p>
	 * Ist die zugrunde liegende DataIdentification nicht zum Senden angemeldet,
	 * erfolgt hier die Anmeldung
	 * </p>
	 *
	 * @param rs
	 *            zu versendender Datensatz
	 * @throws SendSubscriptionNotConfirmed
	 *             wird geworfen, wenn innerhalb der TimeoutZeit keine positive
	 *             Sendesteuerung vom Datenverteiler für die zu versendenden
	 *             Daten vorliegt oder kein Empf&auml;nger vorhanden ist.
	 * @throws OneSubscriptionPerSendData
	 *             wird geworfen, wenn bereits eine lokale Sendeanmeldung für
	 *             die gleichen Daten von einem anderen Anwendungsobjekt
	 *             vorliegt.
	 */
	public synchronized void sendData(final ResultData rs)
			throws SendSubscriptionNotConfirmed, OneSubscriptionPerSendData {
		if (!_IsSendingAllowed) {
			// System.err.println(this);
			subscribe();
		}
		_DavConn.sendData(rs);
		_LastUse = System.currentTimeMillis();
		// __Logger.finest(this.toString() + " Datensatz gesendet.");
	}

	private void waitForSendePermission() {
		_IsTimeOut = true;
		try {
			this.wait(_TimeOut);
		} catch (final InterruptedException ex) {
			ex.printStackTrace();
		}
	}

	/**
	 * Setzt die Zeitspanne, die auf eine positive Sendesteuerung gewartet wird
	 * bevor davon ausgegangen wird, dass es weder Empfänger noch Senke gibt.
	 *
	 * @param timeOut
	 *            int
	 */
	public void setTimeOut(final int timeOut) {
		_TimeOut = timeOut;
	}

	private void subscribe() throws OneSubscriptionPerSendData, SendSubscriptionNotConfirmed {
		subscribeSource();
		// __Logger.fine("nach subscribeSource", this);
		if (_IsTimeOut) {
			throw new SendSubscriptionNotConfirmed("Keine Sendeerlaubnis innnerhalb Timeoutzeit.");
		} else {
			if (!_IsSendingAllowed) {
				if (_State.equals(State.SUBCRIBTION_ERROR)) {
					unsubscribeSender();
					throw new SendSubscriptionNotConfirmed(
							"Sendesteuerung erhielt STOP_SENDING-Befehl. Wahrscheinlich ist kein Empfänger vorhanden.");
				}
				subscribeSender();
				waitForSendePermission();
				if (_IsTimeOut) {
					throw new SendSubscriptionNotConfirmed("Keine Sendeerlaubnis innnerhalb Timeoutzeit.");
				} else {
					if (!_IsSendingAllowed) {
						_State = State.SUBCRIBTION_ERROR;
						throw new SendSubscriptionNotConfirmed("Keine Sendeerlaubnis.");
					}
				}
			}
		}
	}

	private void subscribeSource() {
		SenderSubscribtion.LOGGER.fine("trying to subribe as Source: ", this);
		try {
			this._DavConn.subscribeSender(this, _DataIdentification.getObject(),
					_DataIdentification.getDataDescription(), SenderRole.source());
			_State = State.SUBCRIBED_SOURCE;
			if (!_IsSendingAllowed) {
				waitForSendePermission();
			}
		} catch (final OneSubscriptionPerSendData e) {
			SenderSubscribtion.LOGGER
			.fine("Anmeldung als Quelle nicht möglich; es ist schon eine Quelle/Senke angemeldet: ", this);
		}
	}

	private void subscribeSender() throws OneSubscriptionPerSendData {
		SenderSubscribtion.LOGGER.fine("trying to subribe as Sender: ", this);
		try {
			this._DavConn.subscribeSender(this, _DataIdentification.getObject(),
					_DataIdentification.getDataDescription(), SenderRole.sender());
		} catch (final OneSubscriptionPerSendData e) {
			throw new OneSubscriptionPerSendData(super.toString()
					+ ": Ein Datum kann nur von eine Quelle angemeldet sein (" + _DataIdentification + ").");
		}
		_State = State.SUBCRIBED_SENDER;
	}

	/**
	 * meldet sich als Sender f&uuml;r die DataIdentification ab.
	 */
	public synchronized void unsubscribeSender() {
		this._DavConn.unsubscribeSender(this, _DataIdentification.getObject(),
				_DataIdentification.getDataDescription());
		SenderSubscribtion.LOGGER.fine("unsubscribe");
		_State = State.NOT_INITIALIZED;
		_IsSendingAllowed = false;
	}

	/**
	 * gibt eine Stringrepr&auml;sentation zur&uuml;ck, die die
	 * DataIdentification und den Status enth&auml;lt.
	 *
	 * @return String
	 */
	@Override
	public String toString() {
		final StringBuilder sb = new StringBuilder(super.toString());
		sb.append("[");
		sb.append(_DataIdentification.getObject().getPid()).append(':');
		sb.append(_DataIdentification.getDataDescription().getAttributeGroup().getPid()).append(':');
		sb.append(_DataIdentification.getDataDescription().getAspect().getPid());
		sb.append(" Status: ").append(_State).append(']');
		return sb.toString();
	}

	/**
	 * gibt den Zeitpunkt zur&uuml;ck, zum dem der letzte Datensatz gesendet bzw
	 * das Object erzeugt wurde.
	 *
	 * @return long
	 */
	synchronized public long getLastUse() {
		return _LastUse;
	}
}
