/**
 * Copyright (c) 2006 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * History:
 * 2007-01-29 [KS] __ATTR_NAMEN public gemacht für generischen Zugriff in den
 *                 Auswertefunktionen (Aggregierung...)
 *
 * $Log: AtgGeschwindigkeitenLangzeitdaten.java,v $
 * Revision 1.1  2007/07/04 11:42:17  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.4  2007/06/12 08:49:40  Strey
 * __ATTR_NAMEN für generischen Zugriff auf pblic gesetzt
 *
 * Revision 1.3  2006/11/13 09:24:12  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2006/08/29 08:54:18  Schawaller
 * neue methode: alle ATL als Liste
 *
 * Revision 1.1  2006/08/28 13:43:37  Schawaller
 * Neue klasse zum kapseln von ATG
 *
 */

package de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 *
 * Die abstrakte Klasse stellt Methoden für die Klassen bereit, die
 * Attribut-Gruppen vom Typ Geschwindigkeit kapseln.
 *
 * <p>
 * Copyright: Copyright (c) 2006
 * </p>
 *
 * <p>
 * Organisation: Logos GmbH Rostock (www.logos-ib.de)
 * </p>
 *
 * @author J.Schawaller
 * @version 1.0
 */
public abstract class AtgGeschwindigkeitenLangzeitdaten {

	// PID der Atg wird durch die implementierung der abstrakten methode
	// ermittelt
	private final String ATG_PID_LangZeitDatenGeschwindigkeit = getAtgPid();

	/**
	 * _Data - Daten der ATG
	 */
	private final Data _Data;

	/**
	 * __TYP_ATG_LangzeitdatenGeschwindigkeit - Attributgrupper der Daten
	 */
	private AttributeGroup __TYP_ATG_LangzeitdatenGeschwindigkeit;

	/**
	 * ATT_TYP_LangzeitdatenGeschwindigkeit - Typ der Attribute
	 */
	private static final String ATT_TYP_LANGZEITDATEN_GESCHWINDIGKEIT = "LangzeitdatenGeschwindigkeit";

	/**
	 * ATTR_NAME_KFZ - Attribute Name f&uuml;r Fahrzeugart Kfz
	 */
	public static final String ATTR_NAME_KFZ = "Kfz";

	/**
	 * ATTR_NAME_PKWAE - Attribute Name f&uuml;r Fahrzeugart Pkw&Auml;
	 */
	public static final String ATTR_NAME_PKWAE = "PkwÄ";

	/**
	 * ATTR_NAME_LKWAE - Attribute Name f&uuml;r Fahrzeugart Lkw&Auml;
	 */
	public static final String ATTR_NAME_LKWAE = "LkwÄ";
	/**
	 * ATTR_NAME_KFZNK - Attribute-Name f&uuml;r Fahrzeugart KfzNk
	 */
	public static final String ATTR_NAME_KFZNK = "KfzNk";
	/**
	 * ATTR_NAME_PKWG - Attribute-Name f&uuml;r Fahrzeugart PkwG
	 */
	public static final String ATTR_NAME_PKWG = "PkwG";
	/**
	 * ATTR_NAME_PKW - Attribute-Name f&uuml;r Fahrzeugart Pkw
	 */
	public static final String ATTR_NAME_PKW = "Pkw";
	/**
	 * ATTR_NAME_KRAD - Attribute-Name f&uuml;r Fahrzeugart Krad
	 */
	public static final String ATTR_NAME_KRAD = "Krad";
	/**
	 * ATTR_NAME_LFW - Attribute-Name f&uuml;r Fahrzeugart Lfw
	 */
	public static final String ATTR_NAME_LFW = "Lfw";
	/**
	 * ATTR_NAME_PKWA - Attribute-Name f&uuml;r Fahrzeugart PkwA
	 */
	public static final String ATTR_NAME_PKWA = "PkwA";
	/**
	 * ATTR_NAME_LKW - Attribute-Name f&uuml;r Fahrzeugart Lkw
	 */
	public static final String ATTR_NAME_LKW = "Lkw";
	/**
	 * ATTR_NAME_BUS - Attribute-Name f&uuml;r Fahrzeugart Bus
	 */
	public static final String ATTR_NAME_BUS = "Bus";
	/**
	 * ATTR_NAME_LKWK - Attribute-Name f&uuml;r Fahrzeugart LkwK
	 */
	public static final String ATTR_NAME_LKWK = "LkwK";
	/**
	 * ATTR_NAME_LKWA - Attribute-Name f&uuml;r Fahrzeugart LkwA
	 */
	public static final String ATTR_NAME_LKWA = "LkwA";
	/**
	 * ATTR_NAME_SATTELKFZ - Attribute-Name f&uuml;r Fahrzeugart SattelKfz
	 */
	public static final String ATTR_NAME_SATTELKFZ = "SattelKfz";

	/**
	 * __ATTR_NAMEN - Namen der Attribute als Array zusammengefa&szlig;t
	 */
	public static final String[] ATTR_NAMEN = {
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_KFZ,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_PKWAE,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_LKWAE,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_KFZNK,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_PKWG,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_PKW,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_KRAD,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_LFW,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_PKWA,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_LKW,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_BUS,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_LKWK,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_LKWA,
		AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_SATTELKFZ };

	/**
	 * Erzeugt eine neue Instanz mit dem übergebenem Data.
	 *
	 * @param data
	 *            Data
	 */
	public AtgGeschwindigkeitenLangzeitdaten(final Data data) {
		this._Data = data;
	}

	/**
	 * Erzeugt eine neue Instanz und erstellt ein Data.
	 *
	 */
	public AtgGeschwindigkeitenLangzeitdaten() {
		if (__TYP_ATG_LangzeitdatenGeschwindigkeit == null) {
			__TYP_ATG_LangzeitdatenGeschwindigkeit = LogosClientDavConnection
					.getInstance().getDataModel()
					.getAttributeGroup(ATG_PID_LangZeitDatenGeschwindigkeit);
		}
		_Data = LogosClientDavConnection.getInstance().createData(
				__TYP_ATG_LangzeitdatenGeschwindigkeit);
	}

	/**
	 * Gibt die PID der speziellen ATG wieder.
	 *
	 * @return String PID der Atg
	 */
	public abstract String getAtgPid();

	/**
	 * Initialisiert die Werte des Data für den generichen Zugriff.
	 *
	 */
	public void doInitialize() {
		if (__TYP_ATG_LangzeitdatenGeschwindigkeit == null) {
			__TYP_ATG_LangzeitdatenGeschwindigkeit = LogosClientDavConnection
					.getInstance().getDataModel()
					.getAttributeGroup(ATG_PID_LangZeitDatenGeschwindigkeit);
		}
		final Iterator attrIterator = __TYP_ATG_LangzeitdatenGeschwindigkeit
				.getAttributes().iterator();
		while (attrIterator.hasNext()) {
			final Attribute oAttribute = (Attribute) attrIterator.next();
			if (AtgGeschwindigkeitenLangzeitdaten.ATT_TYP_LANGZEITDATEN_GESCHWINDIGKEIT
					.equals(oAttribute.getAttributeType().getName())) {
				new AtlLangZeitDatenGeschwindigkeit(_Data.getItem(oAttribute
						.getName())).doInitialize();
			}
		} // end while
	}

	/**
	 * Gibt das Data wieder.
	 *
	 * @return Data
	 */
	public Data getData() {
		return _Data;
	}

	/*
	 * #### GETTER - Methoden für den Zugriff
	 */

	/**
	 * Gibt die Daten aller AttributListen als Liste wieder.
	 *
	 * @return List Daten aller AttributListen
	 */
	public List<AtlLangZeitDatenGeschwindigkeit> getAttributListenLangZeitDatenGeschwindigkeit() {
		final List<AtlLangZeitDatenGeschwindigkeit> result = new ArrayList<AtlLangZeitDatenGeschwindigkeit>(
				AtgGeschwindigkeitenLangzeitdaten.ATTR_NAMEN.length);
		for (final String attrName : AtgGeschwindigkeitenLangzeitdaten.ATTR_NAMEN) {
			result.add(new AtlLangZeitDatenGeschwindigkeit(_Data
					.getItem(attrName)));
		}
		return result;
	}

	public AtlLangZeitDatenGeschwindigkeit getKfz() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_KFZ));
	}

	public AtlLangZeitDatenGeschwindigkeit getPkwAE() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_PKWAE));
	}

	public AtlLangZeitDatenGeschwindigkeit getKfzNk() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_KFZNK));
	}

	public AtlLangZeitDatenGeschwindigkeit getPkwG() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_PKWG));
	}

	public AtlLangZeitDatenGeschwindigkeit getPkw() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_PKW));
	}

	public AtlLangZeitDatenGeschwindigkeit getKrad() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_KRAD));
	}

	public AtlLangZeitDatenGeschwindigkeit getLfw() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_LFW));
	}

	public AtlLangZeitDatenGeschwindigkeit getLkwAE() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_LKWAE));
	}

	public AtlLangZeitDatenGeschwindigkeit getPkwA() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_PKWA));
	}

	public AtlLangZeitDatenGeschwindigkeit getLkw() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_LKW));
	}

	public AtlLangZeitDatenGeschwindigkeit getBus() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_BUS));
	}

	public AtlLangZeitDatenGeschwindigkeit getLkwK() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_LKWK));
	}

	public AtlLangZeitDatenGeschwindigkeit getLkwA() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_LKWA));
	}

	public AtlLangZeitDatenGeschwindigkeit getSattelKfz() {
		return new AtlLangZeitDatenGeschwindigkeit(
				_Data.getItem(AtgGeschwindigkeitenLangzeitdaten.ATTR_NAME_SATTELKFZ));
	}

	/**
	 * Gibt die Eigenschaften der Klasse als String wieder.
	 *
	 * @return String Eigenschaften der Klasse
	 */
	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("ATG[").append(getAtgPid()).append("]");
		sb.append(Debug.NEWLINE).append(_Data);
		return sb.toString();
	}

}
