/**
 * Copyright (c) 2006 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * History:
 *
 * $Log: AtlLangZeitDatenFahrzeugAnzahl.java,v $
 * Revision 1.1  2007/07/04 11:43:01  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.1  2006/08/28 13:44:04  Schawaller
 * Neue klasse zum kapseln von ATL
 *
 */

package de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;

/**
 *
 * Die Klasse kapselt die Attribut-Liste
 * <code>atl.langZeitDatenFahrzeugAnzahl</code>.
 *
 * <p>
 * Copyright: Copyright (c) 2006
 * </p>
 *
 * <p>
 * Organisation: Logos GmbH Rostock (www.logos-ib.de)
 * </p>
 *
 * @author J.Schawaller
 * @version 1.0
 */
public class AtlLangZeitDatenFahrzeugAnzahl {

	private final Data _Data;

	public static final String ATTR_NAME_WERT = "Wert";
	public static final String ATTR_NAME_GUETE = "Güte";

	private static final String TYP_ATT_LANG_ZEIT_DATEN_FAHRZEUG_ANZAHL = "att.langZeitDatenFahrzeugAnzahl";

	public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR = "nicht ermittelbar";
	public static final String ZUSTAND_WERT_FEHLERHAFT = "fehlerhaft";
	public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR_FEHLERHAFT = "nicht ermittelbar/fehlerhaft";

	public AtlLangZeitDatenFahrzeugAnzahl(final Data data) {
		_Data = data;
	}

	public void doInitialize() {
		new AtlGuete(
				_Data.getItem(AtlLangZeitDatenFahrzeugAnzahl.ATTR_NAME_GUETE))
		.doInitialize();
		_Data.getScaledValue(AtlLangZeitDatenFahrzeugAnzahl.ATTR_NAME_WERT)
		.setText(
				AtlLangZeitDatenFahrzeugAnzahl.ZUSTAND_WERT_NICHT_ERMITTELBAR);
	}

	public IntegerValueRange getRangeOfLangZeitDatenFahrzeugAnzahl() {
		return ValueRangeHashtable
				.getInstance(this._Data)
				.get(AtlLangZeitDatenFahrzeugAnzahl.TYP_ATT_LANG_ZEIT_DATEN_FAHRZEUG_ANZAHL);
	}

	public int getWert() {
		return _Data.getScaledValue(
				AtlLangZeitDatenFahrzeugAnzahl.ATTR_NAME_WERT).intValue();
	}

	public void setWert(final int wert) {
		_Data.getScaledValue(AtlLangZeitDatenFahrzeugAnzahl.ATTR_NAME_WERT)
		.set(wert);
	}

	public IntegerValueState getZustandWert() {
		return _Data.getScaledValue(
				AtlLangZeitDatenFahrzeugAnzahl.ATTR_NAME_WERT).getState();
	}

	public void setZustandWert(final IntegerValueState integerValueState) {
		_Data.getScaledValue(AtlLangZeitDatenFahrzeugAnzahl.ATTR_NAME_WERT)
		.setState(integerValueState);
	}

	public void setZustandWert(final String zustandsName) {
		_Data.getScaledValue(AtlLangZeitDatenFahrzeugAnzahl.ATTR_NAME_WERT)
		.setText(zustandsName);
	}

	public boolean isState() {
		return _Data.getUnscaledValue(
				AtlLangZeitDatenFahrzeugAnzahl.ATTR_NAME_WERT).isState();
	}

	public boolean isNumber() {
		return _Data.getUnscaledValue(
				AtlLangZeitDatenFahrzeugAnzahl.ATTR_NAME_WERT).isNumber();
	}

	public AtlGuete getGuete() {
		return new AtlGuete(
				_Data.getItem(AtlLangZeitDatenFahrzeugAnzahl.ATTR_NAME_GUETE));
	}

	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		sb.append("atl[")
		.append(AtlLangZeitDatenFahrzeugAnzahl.TYP_ATT_LANG_ZEIT_DATEN_FAHRZEUG_ANZAHL)
		.append("]: ");
		sb.append(" data[").append(_Data).append("]");
		return sb.toString();
	}

}
