/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * History:
 *
 * $Log: ConfigurationObjectWrapper.java,v $
 * Revision 1.2  2007/07/25 13:02:07  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:43:38  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.5  2006/11/13 09:24:12  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.3  2005/11/09 14:23:04  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/02/18 10:08:47  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 13:25:52  Trier
 * <Kein Kommentar eingegeben>
 *
 */

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * BasisKlasse f&uuml;r Klassen, die ConfigurationObjects eines bestimmten
 * ConfigurationObjectTypes wrappen.
 */
abstract public class ConfigurationObjectWrapper extends java.lang.Object {

	public static ConfigurationObjectType __TYP_ANSCHLUSSPUNKT;
	public static ConfigurationObjectType __TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER;
	public static AttributeGroup __ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER;

	/**
	 * das gewrappte ConfigurationObject.
	 */
	protected ConfigurationObject _ConfigurationObject;

	/**
	 * gibt die PID des ConfigurationObjectes zur&uuml;ck.
	 *
	 * @return die PID
	 */
	public final String getPID() {
		return _ConfigurationObject.getPid();
	}

	/**
	 * gibt den Namen des ConfigurationObjectes zur&uuml;ck.
	 *
	 * @return den Namen
	 */
	public final String getName() {
		return _ConfigurationObject.getName();
	}

	/**
	 * gibt das gewrappte ConfigurationObjectes zur&uuml;ck.
	 *
	 * @return die PID
	 */
	public final ConfigurationObject getConfigurationObject() {
		return _ConfigurationObject;
	}

	/**
	 * Implementationen m&uuml;ssen den ConfigurationObjectType
	 * zur&uuml;ckgeben, den die gewrappten ConfigurationObjecte haben.
	 *
	 * @return ConfigurationObjectType der gewrappten ConfigurationObjecte.
	 */
	abstract ConfigurationObjectType getType();

	/**
	 * gibt einen String zum Anzeigen in einem Tree zur&uuml;ck.
	 *
	 * @return den String
	 */
	abstract public String getTreeString();

	/**
	 * gibt einen String zum Anzeigen als ein ToolTip zur&uuml;ck.
	 *
	 * @return den Text
	 */
	public String getToolTipText() {
		return null;
	}

	/**
	 * Implemtationen sollen hier die konfigurierenden AttributGruppen und die
	 * zugeh&ouml;rigen Mengen lesen.
	 *
	 * @param confObject
	 *            ConfigurationObject
	 * @throws DataModellMismatchException
	 *             ungültiges Datenmodell
	 */
	abstract void initData(ConfigurationObject confObject) throws DataModellMismatchException;

	/**
	 * erzeugt eine Instance auf Basis der PID des zu wrappenden Objectes.
	 *
	 * @param pid
	 *            die PID des zu wrappenen Objectes
	 * @throws DataModellMismatchException
	 *             wenn der Typ des ConfigurationsObject nicht durch die Klasse
	 *             gewrappt werden kann oder das Object mit der PID kein
	 *             ConfigurationObject ist.
	 * @throws DataModellObjectNotFoundException
	 *             wenn kein Object mit dieser PID existiert.
	 */
	protected ConfigurationObjectWrapper(final String pid)
			throws DataModellMismatchException, DataModellObjectNotFoundException {
		final SystemObject obj = LogosClientDavConnection.getInstance().getDataModel().getObject(pid);
		if (obj == null) {
			throw new DataModellObjectNotFoundException("Objekt " + pid + " konnte nicht gefunden werden", pid);
		}
		if (!(obj instanceof ConfigurationObject)) {
			throw new DataModellMismatchException("Objekt " + pid + " ist kein ConfigurationObject", pid);
		}
		_ConfigurationObject = (ConfigurationObject) obj;

		if (!_ConfigurationObject.isOfType(this.getType())) {
			throw new DataModellMismatchException("Objekt " + pid + " ist nicht vom typ " + this.getType(), pid);
		} else {
			initData(_ConfigurationObject);
		}
	}

	/**
	 * erzeugt eine Instance, die das angegebene Object wrappt.
	 *
	 * @param confObj
	 *            zu wreppenes ConfigurationObject
	 * @throws DataModellMismatchException
	 */
	protected ConfigurationObjectWrapper(final ConfigurationObject confObj) throws DataModellMismatchException {
		_ConfigurationObject = confObj;
		if (confObj.isOfType(this.getType())) {
			initData(confObj);
		} else {
			throw new DataModellMismatchException("Objekt " + confObj.getPid() + " ist nicht vom typ " + this.getType(),
					confObj.getPid());
		}
	}

	/**
	 * gibt einen String "ObjectType - ObjectName" zur&uuml;ck.
	 *
	 * @return String
	 */
	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer();
		try {
			sb.append(this.getType().getName() + " - ").append(this.getName());
			return sb.toString();
		} catch (final Exception e) {
			return super.toString();
		}
	}

	/**
	 * gibt die KommunikationsPartner zur&uuml;ck, die &uuml;ber die
	 * AnschlussPunkte erreichbar sind.
	 *
	 * @param anschlussPunktGeraeteListe
	 *            List
	 * @return Array der KommunikationsPartner.
	 */
	protected static List getKommunikationPartner(final List anschlussPunktGeraeteListe) {

		if (ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT = (ConfigurationObjectType) model
					.getType("typ.anschlussPunkt");
			ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER = (ConfigurationObjectType) model
					.getType("typ.anschlussPunktKommunikationsPartner");
			ConfigurationObjectWrapper.__ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER = model
					.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");
		}
		final ArrayList<SystemObject> list = new ArrayList<SystemObject>();

		if ((anschlussPunktGeraeteListe != null) && (anschlussPunktGeraeteListe.size() > 0)) {
			for (int apgNr = 0; (apgNr < anschlussPunktGeraeteListe.size()); apgNr++) {
				try {
					final ConfigurationObject apgObject = (ConfigurationObject) anschlussPunktGeraeteListe.get(apgNr);
					if ((apgObject != null) && (apgObject.isOfType(ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT))) {
						final ObjectSet apkpOS = apgObject.getObjectSet("AnschlussPunkteKommunikationsPartner");
						if (apkpOS != null) {
							final List listAPKP = apkpOS.getElements();
							if ((listAPKP != null) && (listAPKP.size() > 0)) {
								for (int apkpNr = 0; apkpNr < listAPKP.size(); apkpNr++) {
									final ConfigurationObject apkpObject = (ConfigurationObject) listAPKP.get(apkpNr);
									if ((apkpObject != null) && (apkpObject.isOfType(
											ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER))) {
										final Data data = apkpObject.getConfigurationData(
												ConfigurationObjectWrapper.__ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER);
										final Data.ReferenceValue refValue = data
												.getReferenceValue("KommunikationsPartner");
										if (refValue != null) {
											final SystemObject so = refValue.getSystemObject();
											list.add(so);
										}
									}
								}
							}
						}
					}
				} catch (final Exception e) {
					e.printStackTrace();
				}
			}
		}
		return list;
	}

	protected static List<SystemObject> getKommunikationPartner(final ConfigurationObject apgObject) {

		if (ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT == null) {
			final DataModel model = LogosClientDavConnection.getInstance().getDataModel();
			ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT = (ConfigurationObjectType) model
					.getType("typ.anschlussPunkt");
			ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER = (ConfigurationObjectType) model
					.getType("typ.anschlussPunktKommunikationsPartner");
			ConfigurationObjectWrapper.__ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER = model
					.getAttributeGroup("atg.anschlussPunktKommunikationsPartner");

			final List<SystemObject> list = ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER
					.getObjects();
			model.getConfigurationData(list, ConfigurationObjectWrapper.__ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER);
		}
		final ArrayList<SystemObject> list = new ArrayList<SystemObject>();

		try {
			if ((apgObject != null) && (apgObject.isOfType(ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT))) {
				final ObjectSet apkpOS = apgObject.getObjectSet("AnschlussPunkteKommunikationsPartner");
				if (apkpOS != null) {
					final List listAPKP = apkpOS.getElements();
					if ((listAPKP != null) && (listAPKP.size() > 0)) {
						for (int apkpNr = 0; apkpNr < listAPKP.size(); apkpNr++) {
							final ConfigurationObject apkpObject = (ConfigurationObject) listAPKP.get(apkpNr);
							if ((apkpObject != null) && (apkpObject
									.isOfType(ConfigurationObjectWrapper.__TYP_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER))) {
								final Data data = apkpObject.getConfigurationData(
										ConfigurationObjectWrapper.__ATG_ANSCHLUSSPUNKT_KOMMUNIKATIONSPARTNER);
								final Data.ReferenceValue refValue = data.getReferenceValue("KommunikationsPartner");
								if (refValue != null) {
									final SystemObject so = refValue.getSystemObject();
									list.add(so);
								}
							}
						}
					}
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();
		}
		return list;
	}

}
