/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * History:
 *
 * $Log: SteuerModul.java,v $
 * Revision 1.2  2007/07/25 13:02:08  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:44:12  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.7  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.6  2005/11/09 14:23:04  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.5  2005/10/05 11:27:55  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2005/04/21 09:15:34  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/01/31 15:24:48  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/25 09:43:11  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 13:26:06  Trier
 * <Kein Kommentar eingegeben>
 *
 */

import java.util.ArrayList;
import java.util.List;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellMismatchException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.DataModellObjectNotFoundException;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection;

/**
 * kapselt den DatenModellTyp steuerModul
 */
public class SteuerModul extends ConfigurationObjectWrapper {

	private static WeakHashMap<String, SteuerModul> __Cache = new WeakHashMap<String, SteuerModul>();

	private static ConfigurationObjectType __ConfigurationObjectType;
	private static ConfigurationObjectType __ConfigurationObjectTypeOfEAK;
	private static AttributeGroup __AtgGeraet;

	private String bezeichnung;
	private int knotenNummer;
	private String umsetzungsModul;

	private ArrayList<EAK> eakList;

	private DEKanal deKanal;
	private KriBus kriBus;

	/**
	 * setzt den Bus &uuml;ber den das SteuerModul am Kri angeschlossen ist.
	 */
	public void setBus(final KriBus bus) {
		kriBus = bus;
	}

	/**
	 * gibt den Bus zur&uuml;ck, &uuml;ber den das SteuerModul am Kri
	 * angeschlossen ist.
	 */
	public KriBus getBus() {
		return kriBus;
	}

	/**
	 * gibt die Bezeichnung des EAK zur&uuml;ck.
	 *
	 * @return String
	 */
	public String getBezeichnung() {
		return bezeichnung;
	}

	/**
	 * gibt den DE-Kanal zum Steuern des SteuerModuls zur&uuml;ck.
	 *
	 * @return DEKanal
	 */
	public DEKanal getSteuerDEKanal() {
		return deKanal;
	}

	/**
	 * gibt die Knotennummer (OSI-7-Adresse) des Ger&auml;tes zur&uuml;ck.
	 *
	 * @return die Knotennummer;
	 */
	public int getKnotenNummer() {
		return knotenNummer;
	}

	/**
	 * gibt das für die Umsetzung des Protokolls (Lesen und Schreiben)
	 * verwendete Modul an diesem Knoten zur&uuml;ck.
	 *
	 * @return Klassennamen des UmsetzungsModuls.
	 */
	public String getUmsetzungsModul() {
		return umsetzungsModul;
	}

	/**
	 * gibt die EAK's zur&uuml;ck oder null, wenn keine angeschlossen sind.
	 *
	 * @return EAK's.
	 */
	public EAK[] getEAKs() {
		if ((eakList != null) && (eakList.size() > 0)) {
			return eakList.toArray(new EAK[eakList.size()]);
		} else {
			return null;
		}
	}

	/**
	 * <p>
	 * gibt den Sammelkanal des entsprechenen Typs zur&uuml;ck
	 * </p>
	 *
	 * @param classOfdeKanal
	 *            {@link DEKanal.Lve}, {@link DEKanal.Axl}, {@link DEKanal.Ufd},
	 *            {@link DEKanal.Wzg} oder {@link DEKanal.Vlt},
	 * @return Sammelkanl f&uuml;r entsprechene Funktionsgruppe
	 */
	public DEKanal getSammelKanal(final Class classOfdeKanal) {

		DEKanal deKanal = null;
		for (int eakNr = 0; ((eakNr < eakList.size()) && (deKanal == null)); eakNr++) {
			final EAK eak = eakList.get(eakNr);
			final DEKanal[] deArray = eak.getDEKanals();
			for (int deNr = 0; ((deNr < deArray.length) && (deKanal == null)); deNr++) {
				final DEKanal tmpDEKanal = deArray[deNr];
				if ((tmpDEKanal.getDEKanal() == 255)
						&& (classOfdeKanal.isInstance(tmpDEKanal))) {
					deKanal = tmpDEKanal;
				}
			}
		}
		return deKanal;
	}

	/**
	 * <p>
	 * gibt alle Sammelkan&auml;le zur&uuml;ck, die an dem Steuermodul
	 * angeschlossen sind.
	 * </p>
	 *
	 * @return DEKanal[]
	 */
	public DEKanal[] getSammelKanaele() {
		final ArrayList<DEKanal> list = new ArrayList<DEKanal>(1);
		for (int eakNr = 0; eakNr < eakList.size(); eakNr++) {
			final EAK eak = eakList.get(eakNr);
			final DEKanal[] deArray = eak.getDEKanals();
			for (final DEKanal tmpDEKanal : deArray) {
				if (tmpDEKanal.getDEKanal() == 255) {
					list.add(tmpDEKanal);
				}
			}
		}
		return list.toArray(new DEKanal[list.size()]);
	}

	/**
	 * erzeugt das WrapperObject f&uuml;r das Object mit der angegebenen Pid
	 *
	 * @see ConfigurationObjectWrapper#ConfigurationObjectWrapper(String pid)
	 *
	 * @param pid
	 *            String
	 * @throws DataModellMismatchException
	 * @throws DataModellObjectNotFoundException
	 */
	protected SteuerModul(final String pid) throws DataModellMismatchException,
	DataModellObjectNotFoundException {
		super(pid);
	}

	/**
	 * erzeugt das WrapperObject f&uuml;r das angegebene Object.
	 *
	 * @see ConfigurationObjectWrapper#ConfigurationObjectWrapper(ConfigurationObject
	 *      confObj)
	 *
	 * @param confObj
	 *            ConfigurationObject, das ein Steuermodul repr&auml;sentiert.
	 * @throws DataModellMismatchException
	 */
	protected SteuerModul(final ConfigurationObject confObj)
			throws DataModellMismatchException {
		super(confObj);
	}

	@Override
	protected void initData(final ConfigurationObject confObj)
			throws DataModellMismatchException {

		// virtueller EAK (254 OsiAddr==0)
		EAK virtEAK = null;

		final DataModel model = LogosClientDavConnection.getInstance()
				.getDataModel();
		// if (__AtgGeraet == null) {
		// __AtgGeraet = model.getAttributeGroup("atg.gerät");
		// __ConfigurationObjectTypeOfEAK =
		// (ConfigurationObjectType)model.getType("typ.eak");
		// }
		final Data oData = confObj
				.getConfigurationData(SteuerModul.__AtgGeraet);
		if (oData != null) {
			bezeichnung = oData.getTextValue("Bezeichnung").getText();
			knotenNummer = oData.getUnscaledValue("KnotenNummer").intValue();
			umsetzungsModul = oData.getTextValue("UmsetzungsModul").getText();
		}
		final ObjectSet os = confObj.getObjectSet("Eak");
		if (os != null) {
			final List list = os.getElements();
			if ((list != null) && (list.size() > 0)) {
				eakList = new ArrayList<EAK>(list.size());
				for (int i = 0; i < list.size(); i++) {
					final ConfigurationObject eakObj = (ConfigurationObject) list
							.get(i);
					if (eakObj.getType().equals(
							SteuerModul.__ConfigurationObjectTypeOfEAK)) {
						final EAK eak = new EAK(eakObj);
						eak.setSteuerModul(this);
						eakList.add(eak);
						if (eak.getOSI2Adresse() == 0) {
							virtEAK = eak;
						}
					}
				}
			}
		}
		// Zuordnung der Kontroll-DEKanale zum SM bzw. den EAK
		if (virtEAK != null) {
			for (int i = 0; i < eakList.size(); i++) {
				final EAK eak = eakList.get(i);
				if (eak != virtEAK) {
					final DEKanal deKanal = virtEAK.findDEKanal(eak
							.getOSI2Adresse());
					eak.setDEKanal(deKanal);
				}
			}
		}
		deKanal = virtEAK.findDEKanal(0);
		SteuerModul.__Cache.put(this.getPID(), this);
	}

	@Override
	ConfigurationObjectType getType() {
		if (SteuerModul.__ConfigurationObjectType == null) {
			final DataModel model = LogosClientDavConnection.getInstance()
					.getDataModel();
			SteuerModul.__ConfigurationObjectType = (ConfigurationObjectType) model
					.getType("typ.steuerModul");
			SteuerModul.__AtgGeraet = model.getAttributeGroup("atg.gerät");
			SteuerModul.__ConfigurationObjectTypeOfEAK = (ConfigurationObjectType) model
					.getType("typ.eak");

			final List<SystemObject> list = SteuerModul.__ConfigurationObjectType
					.getObjects();
			model.getConfigurationData(list, SteuerModul.__AtgGeraet);
		}
		return SteuerModul.__ConfigurationObjectType;
	}

	@Override
	public String getTreeString() {
		return getName();
	}

	/**
	 * gibt ein Array aller Objekte zurück, die vom Typ typ.steuerModul sind.
	 *
	 * @return Unterzentrale[]
	 */
	public static SteuerModul[] getAll() throws DataModellMismatchException {
		final List list = LogosClientDavConnection.getInstance().getDataModel()
				.getType("typ.steuerModul").getElements();
		final SteuerModul[] retValue = new SteuerModul[list.size()];
		for (int i = 0; i < list.size(); i++) {
			final ConfigurationObject confObj = (ConfigurationObject) list
					.get(i);
			final SteuerModul sm = SteuerModul.__Cache.get(confObj.getPid());
			if (sm == null) {
				retValue[i] = new SteuerModul((ConfigurationObject) list.get(i));
			} else {
				retValue[i] = sm;
			}
		}
		return retValue;
	}

	/**
	 * gibt das SteuerModul mit der &uuml;bergebenen Pid zur&uuml;ck oder null,
	 * wenn es kein SteuerModul mit der Pid gibt.
	 *
	 * @param pid
	 *            String
	 * @throws DataModellMismatchException
	 * @return SteuerModul
	 */
	public static SteuerModul getInstance(final String pid)
			throws DataModellMismatchException {
		SteuerModul sm = SteuerModul.__Cache.get(pid);
		if (sm == null) {
			final Object object = LogosClientDavConnection.getInstance()
					.getDataModel().getObject(pid);
			if ((object != null) && (object instanceof ConfigurationObject)) {
				sm = new SteuerModul((ConfigurationObject) object);
			}
		}
		return sm;
	}

}
