/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output;

/**
 * History:
 *
 * $Log: AbstractDataChannelBASt.java,v $
 * Revision 1.4  2007/07/26 12:49:55  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2007/07/26 08:01:42  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2007/07/04 11:48:42  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:27  Trier
 * dvs alles
 *
 * Revision 1.5  2006/11/13 09:25:26  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2005/09/22 06:56:26  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/02/28 14:40:47  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/02/24 07:01:01  Trier
 * Variante ohne Protokoll
 *
 * Revision 1.1  2005/01/21 14:39:59  Trier
 * <Kein Kommentar eingegeben>
 *
 */

import java.text.DecimalFormat;
import java.util.Calendar;
import java.util.Date;

import de.bsvrz.dav.daf.main.Dataset;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.ResultDataChannel;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.LangZeitZaehlStelle;

/**
 * stellt die Funktionalit&auml;t bereit die von allen Channels zur Ausgabe von
 * BASt-Formaten ben&ouml;tigt werden.
 *
 * <p>
 * Vor der Bearbeitung der eigentlichen Daten durch die Aufrufe der Methode
 * {@link ResultDataChannel#processRecord(Dataset[], Calendar)}, muss der
 * Channel vollst&auml;ndig initialisiert sein.
 * </p>
 *
 * @version 1.0
 */
abstract public class AbstractDataChannelBASt extends FileDataChannel {

	/**
	 * DecimalFormat initialisiert mit "000"
	 */
	public static DecimalFormat THREE_DIGIT_FORMAT = new DecimalFormat("000");
	/**
	 * DecimalFormat initialisiert mit "00"
	 */
	public static DecimalFormat TWO_DIGIT_FORMAT = new DecimalFormat("00");
	/**
	 * Array f&uuml;r MonatsCodes 1=Jan .. c=Dez (laut Tabelle der BASt)
	 */
	protected static final char[] MONATE = new char[] { '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c' };

	ChannelLogger _Logger;

	/**
	 * aktuelle Langzeitz&auml;hlstelle
	 */
	protected LangZeitZaehlStelle _ZaehlStelle;

	/**
	 * Anzahl der Fahrstreifen der aktuellen Z&auml;hlstelle, wird in
	 * setZaehlstelle berechnet.
	 */
	protected int _CountOfFS;

	/**
	 * die letzte bearbeitete Stunde (um Sommer-/Winterzeitumstellung zu
	 * behandeln).
	 */
	protected int _LastHour = -1;

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link #setZaehlstelle(LangZeitZaehlStelle zaehlstelle) setZaehlstelle()}
	 * initialisiert werden muss.
	 */
	public AbstractDataChannelBASt() {
		super();
	}

	/**
	 * erzeugt einen DataChannel, der vor dem Aufruf von
	 * {@link #processRecord(Dataset[], Calendar) processRecord()} mit
	 * setZaehlStelle() initialisiert werden muss.
	 *
	 * <p>
	 * Ist die Datei vorhanden wird diese &uuml;berschrieben.
	 * </p>
	 *
	 * @param fileName
	 *            Dateiname mit vollst&auml;ndigem Pfad
	 */
	public AbstractDataChannelBASt(final String fileName) {
		super(fileName);
	}

	/**
	 * <p>
	 * setzt die Z&auml;hlstelle.
	 * </p>
	 *
	 * @param zaehlstelle
	 *            LangZeitZaehlStelle
	 * @throws ChannelException
	 *             Fehler beim Zugriff auf den Channel
	 */
	abstract public void setZaehlstelle(LangZeitZaehlStelle zaehlstelle) throws ChannelException;

	/**
	 * <p>
	 * ist der Channel ge&ouml;ffnet, werden gepufferte Daten in die Datei
	 * geschrieben und der Channel zur Ausgabe in die Datei neu ge&ouml;ffnet,
	 * dessen Namen durch die Implementation der Methode
	 * {@link #constructFilename(LangZeitZaehlStelle zaehlStelle, String path, int year, int monthOrQuartal, boolean isQuartal)
	 * constructFilename} aus den Daten der Z&auml;hlstelle, Jahr, Monat bzw
	 * Quartal bestimmt wird.
	 * </p>
	 *
	 * @param zaehlStelle
	 *            LangZeitZaehlStelle dessen Daten ausgegeben werden sollen.
	 * @param path
	 *            Pfad in dem die Datei angelegt wird.
	 * @param year
	 *            das Jahr f&uuml;r das die Datei erzeugt werden soll.
	 * @param monthOrQuartal
	 *            der Monat bzw. das Quartal f&uuml;r das die Datei erzeugt
	 *            werden soll.
	 * @param isQuartal
	 *            true, wenn monthOrQuartal als Quartal interpretiert werden
	 *            soll.
	 * @throws ChannelException
	 *             Fehler beim Zugriff auf den Channel
	 */
	public void reset(final LangZeitZaehlStelle zaehlStelle, String path, final int year, final int monthOrQuartal,
			final boolean isQuartal) throws ChannelException {
		if (!path.endsWith(System.getProperty("file.separator"))) {
			path = path + System.getProperty("file.separator");
		}
		setZaehlstelle(zaehlStelle);
		reset(constructFilename(zaehlStelle, path, year, monthOrQuartal, isQuartal));
	}

	/**
	 * <p>
	 * ist der Channel ge&ouml;ffnet, werden gepufferte Daten in die Datei
	 * geschrieben und der Channel zur Ausgabe in die Datei neu ge&ouml;ffnet,
	 * dessen Namen durch die Implementaiton der Methode
	 * {@link #constructFilename(LangZeitZaehlStelle zaehlStelle, String path, int year, int startDayOfYear, int endDayOfYear)
	 * constructFilename} aus den Daten der Z&auml;hlstelle, Jahr, Monat bzw
	 * Quartal bestimmt wird.
	 * </p>
	 *
	 * @param zaehlStelle
	 *            LangZeitZaehlStelle dessen Daten ausgegeben werden sollen.
	 * @param path
	 *            Pfad in dem die Datei angelegt wird.
	 * @param year
	 *            das Jahr f&uuml;r das die Datei erzeugt werden soll.
	 * @param startDayOfYear
	 *            der erste Tag, f&uuml;r den die Daten ausgegeben werden
	 *            (1-366).
	 * @param endDayOfYear
	 *            der letzte Tag, f&uuml;r den die Daten ausgegeben werden
	 *            (1-366).
	 * @throws ChannelException
	 *             Fehler beim Zugriff auf den Kanal
	 */
	public void reset(final LangZeitZaehlStelle zaehlStelle, String path, final int year, final int startDayOfYear,
			final int endDayOfYear) throws ChannelException {
		if (!path.endsWith(System.getProperty("file.separator"))) {
			path = path + System.getProperty("file.separator");
		}
		setZaehlstelle(zaehlStelle);
		reset(constructFilename(zaehlStelle, path, year, startDayOfYear, endDayOfYear));
	}

	/**
	 * <p>
	 * ist der Channel geöffnet, werden gepufferte Daten in die Datei
	 * geschrieben und der Channel zur Ausgabe in die angegebenen Datei neu
	 * ge&ouml;ffnet.
	 * </p>
	 *
	 * @param zaehlStelle
	 *            LangZeitZaehlStelle dessen Daten ausgegeben werden sollen.
	 * @param fileName
	 *            die Datei, in die geschrieben werden soll.
	 * @throws ChannelException
	 *             Fehler beim Zugriff auf den Kanal
	 */
	public void reset(final LangZeitZaehlStelle zaehlStelle, final String fileName) throws ChannelException {
		setZaehlstelle(zaehlStelle);
		reset(fileName);
	}

	/**
	 * <p>
	 * Hilfsmethode, die pr&uuml;ft, ob das ResulData[] irgendwelche Data hat.
	 * </p>
	 * <p>
	 * Funktion wird verwendet, um zu pr&uuml;fen, ob ein Totalausfall einer
	 * Z&auml;hlstelle vorliegt.
	 *
	 * @param rsArray
	 *            zu pr&uuml;fendes ResultData[]
	 * @return true, wenn das Array hasData, sonst false.
	 */
	public static boolean hasData(final Dataset[] rsArray) {
		boolean retValue = false;
		if ((rsArray != null) && (rsArray.length > 0)) {
			for (int i = 0; ((i < rsArray.length) && (!retValue)); i++) {
				retValue = ((rsArray[i] != null) && (rsArray[i].getData() != null));
			}
		}
		return retValue;
	}

	/**
	 * gibt den Dateinamen zur&uuml;ck, der sich aus den Daten der
	 * Z&auml;hlstelle, dem Jahr, dem Anfangstag und Endtag ergibt.
	 *
	 * @return vollst&auml;ndig klassifizierter Dateiname
	 */
	abstract protected String constructFilename(LangZeitZaehlStelle zaehlStelle, String path, int year,
			int startDayOfYear, int endDayOfYear);

	/**
	 * gibt den Dateinamen zur&uuml;ck, der sich aus den Daten der
	 * Z&auml;hlstelle, dem Jahr, dem Monat bzw Quartal ergibt.
	 *
	 * @return vollst&auml;ndig klassifizierter Dateiname
	 */
	abstract protected String constructFilename(LangZeitZaehlStelle zaehlStelle, String path, int year,
			int monthOrQuartal, boolean isQuartal);

	public void setChannelLogger(final ChannelLogger logger) {
		_Logger = logger;
	}

	/**
	 * f&uuml;gt eine Warnung zum Protokoll hinzu.
	 *
	 * @param date
	 *            Date
	 * @param message
	 *            String
	 */
	public void logWarning(final Date date, final String message) {
		if (_Logger != null) {
			_Logger.logWarning(date, message);
		}
	}

	/**
	 * f&uuml;gt ein Fehler zum Protokoll hinzu.
	 *
	 * @param date
	 *            Calendar
	 * @param message
	 *            Object
	 * @param throwable
	 *            Throwable
	 */
	public void logError(final Date date, final String message, final Throwable throwable) {
		if (_Logger != null) {
			_Logger.logError(date, message, throwable);
		}

	}

}
