/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output;

import java.util.Calendar;

import de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper.AtgVerkehrsdatenLangZeitIntervall;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper.AtlGeschwindigkeit;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper.AtlGuete;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper.AtlVerkehrsStaerkeStunde;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.LangZeitZaehlStelle;

/**
 * History:
 *
 * $Log: DataChannelBASt_MV_Test.java,v $
 * Revision 1.6  2007/08/22 07:48:45  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.5  2007/07/26 12:49:55  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2007/07/26 08:01:42  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2007/07/25 13:02:06  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2007/07/04 11:48:42  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:27  Trier
 * dvs alles
 *
 * Revision 1.1  2007/01/16 09:56:49  Trier
 * Erweiterung BastImport/Export
 * insbesondere die Unterstützung  unterschiedl. Aspekte
 *
 *
 */

/**
 * <p>
 * Channel zur Ausgabe von Daten in das BASt-Bestandsbandformat in der Version,
 * wie sie bei Ausgabe aus der Access-DB in MV benutzt wird.
 * </p>
 *
 * <p>
 * Es wird die Attributegrupe atg.verkehrsDatenLangZeitIntervall mit dem Aspekt
 * asp.messWertErsetzung ausgegeben.
 * </p>
 * <p>
 * Die Unterstützung f&uuml;r die durch die BASt definierten Pr&uuml;fzeichen
 * ist der folgenen Tabelle zu entnehmen.
 * </p>
 * <table border=1 align=center cellpadding=2 cellspacing=0 summary="Werte">
 * <tr>
 * <th align=left>Pr&uuml;fkennzeichen</th> <th align=left>Bemerkung</th> </tr>
 * <tr> <td>&nbsp;</td> <td>korrekte, regelmäßige Werte</td> </tr> <tr>
 * <td>'</td> <td>alle anderen Werte</td> </tr> </table>
 *
 * <p> Bei der Umstellung MESZ zur MEZ wird der 2. Datensatz für 02 Uhr
 * ausgegeben. </p>
 *
 * <p> siehe auch:
 * {@link de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output.AbstractDataChannelBASt
 * AbstractDataChannelBASt} </p>
 *
 */
public class DataChannelBASt_MV_Test extends DataChannelBAStV1_X {

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link DataChannelBAStV1_X#setZaehlstelle(LangZeitZaehlStelle)}
	 * initialisiert werden muss.
	 */
	public DataChannelBASt_MV_Test() {
		super();
		setCountOfGeschwKlassen(10);
	}

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link DataChannelBAStV1_X#setZaehlstelle(LangZeitZaehlStelle)}
	 * initialisiert werden muss zur Ausgabe der angegebenen SatzArt.
	 */
	public DataChannelBASt_MV_Test(final int satzArt) {
		super(satzArt);
		setCountOfGeschwKlassen(10);
	}

	/**
	 * erzeugt einen DataChannel, der vor dem Aufruf von
	 * {@link #processRecord(de.bsvrz.dav.daf.main.Dataset[], Calendar)} mit
	 * {@link DataChannelBAStV1_X#setZaehlstelle(LangZeitZaehlStelle)}
	 * initialisiert werden muss.
	 *
	 * <p>
	 * Ist die Datei vorhanden wird diese &uuml;berschrieben.
	 * </p>
	 *
	 * @param fileName
	 *            Dateiname mit vollst&auml;ndigem Pfad
	 */
	public DataChannelBASt_MV_Test(final String fileName) throws ChannelException {
		super(fileName);
		setCountOfGeschwKlassen(10);
	}

	/**
	 * erzeugt den f&uuml;r eine Z&auml;hlstelle konstanten Zeilenanfang. Der
	 * Anfang jeder Zeile besteht aus TK-BlattNr, Z&auml;hlstellenNr.,
	 * L&auml;nderkennung.
	 */
	@Override
	protected String calculateLineHeader() {
		final StringBuffer lineHeader = new StringBuffer(8);
		lineHeader.append(_ZaehlStelle.getTK25AsString());
		lineHeader.append(_ZaehlStelle.getBastNummerAsString());
		return lineHeader.toString();
	}

	@Override
	protected ValueExtractor getValueExtractor8plus1() {
		return new ValueExtractor8plus1();
	}

	/**
	 * extrahiert die Werte aus den ResultData und baut einen String mit den
	 * Werten f&uuml;r die Klassifizierungsgruppe 8plus1.
	 */
	private class ValueExtractor8plus1 implements ValueExtractor {

		@Override
		public StringBuffer extractDataKSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray) {

			final StringBuffer sb = new StringBuffer();
			// Werte: nkKfz, Krad, Pkw, LfW, PkwA, Lkw, LkwA, SattelKfz, Busse
			// für alle Fahrstreifen
			for (int i = 0; i < _CountOfFS; i++) {
				if ((atgLZArray == null) || (atgLZArray[i] == null)) {
					sb.append("a0000a0000a0000a0000a0000a0000a0000a0000a0000");
				} else {
					sb.append(formatBastST(atgLZArray[i].getqKfzNk()));
					sb.append(formatBastST(atgLZArray[i].getqKrad()));
					sb.append(formatBastST(atgLZArray[i].getqPkw()));
					sb.append(formatBastST(atgLZArray[i].getqPkwA()));
					sb.append(formatBastST(atgLZArray[i].getqBus()));
					sb.append(formatBastST(atgLZArray[i].getqLfw()));
					sb.append(formatBastST(atgLZArray[i].getqLkw()));
					sb.append(formatBastST(atgLZArray[i].getqLkwA()));
					sb.append(formatBastST(atgLZArray[i].getqSattelKfz()));

				}
			}
			return sb;
		}

	}

	@Override
	protected StringBuffer extractDataGSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray) {

		final StringBuffer sb = new StringBuffer();
		// Werte: vPkwÄ, v85PkwÄ, sPkwÄ, vLkwÄ, sLkwÄ für alle Fahrstreifen
		for (int i = 0; i < _CountOfFS; i++) {
			if ((atgLZArray == null) || (atgLZArray[i] == null)) {
				sb.append("'    '    '    '    '    ");
			} else {
				sb.append(formatBastST(atgLZArray[i].getvPkwAE()));
				sb.append(formatBastST(atgLZArray[i].getv85PkwAE()));
				sb.append(formatBastST(atgLZArray[i].getsPkwAE()));
				sb.append(formatBastST(atgLZArray[i].getvLkwAE()));
				sb.append(formatBastST(atgLZArray[i].getsLkwAE()));
			}
		}
		return sb;
	}

	@Override
	protected StringBuffer extractDataMSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray)
			throws ChannelException {
		StringBuffer retValue = null;
		try {
			// Bestimmen der Anzahl der Klassen
			if (countOfGeschwKlassen == 0) {
				for (int i = 0; ((i < _CountOfFS) && (countOfGeschwKlassen == 0)); i++) {
					final AtlVerkehrsStaerkeStunde[] atlArray = atgLZArray[i].getqPkwAEGeschwKlassen();
					if ((atlArray != null) && (atlArray.length > 0)) {
						_CountOfFS = atlArray.length;
						retValue = extractDataMSatz(atgLZArray);
					}
				}
			} else {
				retValue = new StringBuffer();
				for (int i = 0; i < _CountOfFS; i++) {
					AtlVerkehrsStaerkeStunde[] atlPkwArray = null;
					AtlVerkehrsStaerkeStunde[] atlLkwArray = null;
					if (atgLZArray[i] != null) {
						atlPkwArray = atgLZArray[i].getqPkwAEGeschwKlassen();
						atlLkwArray = atgLZArray[i].getqLkwAEGeschwKlassen();
					}
					if ((atlPkwArray != null) && (atlPkwArray.length == countOfGeschwKlassen)) {
						for (int kl = 0; kl < countOfGeschwKlassen; kl++) {
							retValue.append(formatBastST(atlPkwArray[kl]));
						}
					} else {

						if ((atlPkwArray != null) && (atlPkwArray.length != countOfGeschwKlassen)) {

							logWarning(chCalendar.getTime(),
									"Einteilung der Geschwindigkeitsklassen für Pkw des " + i
											+ ". Fahrstreifen stimmt nicht (Ist: " + atlPkwArray.length + "  Soll: "
											+ countOfGeschwKlassen + ").");

						}

						for (int kl = 0; kl < countOfGeschwKlassen; kl++) {
							retValue.append("'    ");
						}
					}
					if ((atlLkwArray != null) && (atlLkwArray.length == countOfGeschwKlassen)) {
						for (int kl = 0; kl < countOfGeschwKlassen; kl++) {
							retValue.append(formatBastST(atlLkwArray[kl]));
						}
					} else {

						if ((atlLkwArray != null) && (atlLkwArray.length != countOfGeschwKlassen)) {
							logWarning(chCalendar.getTime(),
									"Einteilung der Geschwindigkeitsklassen für Lkw des " + i
											+ ". Fahrstreifen stimmt nicht (Ist: " + atlLkwArray.length + "  Soll: "
											+ countOfGeschwKlassen + ").");
						}

						for (int kl = 0; kl < countOfGeschwKlassen; kl++) {
							retValue.append("'    ");
						}
					}
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();
			throw new ChannelException("Fehler beim Extrahieren der Daten für dem M-Satz, Zeile wurde geschrieben.", e);
		}
		return retValue;
	}

	@Override
	protected StringBuffer extractDataVSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray) {
		final StringBuffer sb = new StringBuffer();
		// Werte: nkKfz, Krad, Pkw, LfW, PkwA, Lkw, LkwA, SattelKfz, Busse für
		// alle Fahrstreifen
		for (int i = 0; i < _CountOfFS; i++) {
			if ((atgLZArray == null) || (atgLZArray[i] == null)) {
				sb.append("'    '    ");
			} else {
				sb.append(formatBastST(atgLZArray[i].getqKfz()));
				sb.append(formatBastST(atgLZArray[i].getqLkwAE()));
			}
		}
		return sb;
	}

	protected String formatBastST(final AtlVerkehrsStaerkeStunde atl) {
		String retValue;
		if (atl.isState()) {
			retValue = "a0000";
		} else {
			final AtlGuete guete = atl.getGuete();
			// if (guete.isNumber() && (guete.getIndex()>0.9999999)) {
			if (guete.isNumber()) {
				final StringBuffer sb = new StringBuffer(5);
				sb.append(atl.getWert());
				while (sb.length() < 4) {
					sb.insert(0, '0');
				}
				sb.insert(0, '-');
				retValue = sb.toString();
			} else {
				retValue = "a0000";
			}
		}
		return retValue;
	}

	private String formatBastST(final AtlGeschwindigkeit atl) {
		String retValue;
		if (atl.isState()) {
			retValue = "a    ";
		} else {
			final AtlGuete guete = atl.getGuete();
			// if (guete.isNumber() && (guete.getIndex()>0.9999999)) {
			if (guete.isNumber()) {
				final StringBuffer sb = new StringBuffer(5);
				sb.append(atl.getWert() * 10).append('0');
				while (sb.length() < 4) {
					sb.insert(0, '0');
				}
				sb.insert(0, '-');
				retValue = sb.toString();
			} else {
				retValue = "'a   ";
			}
		}
		return retValue;
	}

}
