/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.funclib.logger;

/**
 * History:
 *
 * $Log: LoggerFactory.java,v $
 * Revision 1.2  2007/08/06 09:50:19  Trier
 * Änderung Log-System
 *
 * Revision 1.1.1.1  2007/06/14 08:37:53  Trier
 * Allgemeine Funktionsbibliothek
 *
 */

/**
 * <p>
 * Die LoggerFactory erm&ouml;glicht den Zugriff auf das Log-System ohne das
 * konkrete Log-Framework zu kennen.
 * </p>
 * <p>
 * Die Methode {@link LoggerFactory#getLoggerFactory()} gibt eine konkrete
 * LoggerFactory zur&uuml;ck.
 * </p>
 * <p>
 * <b>Bestimmung der konkreten Factory:</b>
 * </p>
 * <ul style="margin-top:0">
 * <li>Klasse <code>de.logosib.dvs.funclib.logger.LoggerFactory</code> vorhanden
 * ist =&gt; LogSystem DatenVerteiler</li>
 * <li>Klasse <code>de.logosib.funclib.log4.LoggerFactory</code> vorhanden ist
 * =&gt; LogSystem log4j</li>
 * <li><code>sonst</code> =&gt; {@link StandardLoggerFactory}, die
 * java.util.logging verwendet.</li>
 * </ul>
 * <p>
 * Ein Logger-Object ist mit einer der folgenden Methoden zu holen:
 *
 * <pre style="margin-top:0">
 *    {@link LoggerFactory#getLogger() de.logosib.funclib.logger.LoggerFactory.getLoggerFactory().getLogger()}
 *    {@link LoggerFactory#getLogger(Class c) de.logosib.funclib.logger.LoggerFactory.getLoggerFactory().getLogger(Class c)}
 *    {@link LoggerFactory#getLogger(String name) de.logosib.funclib.logger.LoggerFactory.getLoggerFactory().getLogger(String name)}
 * </pre>
 *
 * @author Ralf Trier
 * @version 1.0
 */
public abstract class LoggerFactory {

	private static LoggerFactory loggerFactory;

	static {
		if (LoggerFactory.loggerFactory == null) {
			LoggerFactory.loggerFactory = LoggerFactory
					.createLoggerFactory("de.logosib.dvs.funclib.logger.LoggerFactory");

			if (LoggerFactory.loggerFactory != null) {
				System.err.println("Es wird das LogSystem vom Datenverteiler verwendet (Factory="
						+ LoggerFactory.loggerFactory.getClass() + ").");
			} else {
				LoggerFactory.loggerFactory = LoggerFactory
						.createLoggerFactory("de.logosib.funclib.log4.LoggerFactory");
				if (LoggerFactory.loggerFactory != null) {
					System.err.println(
							"Logging erfolgt mit log4j (Factory=" + LoggerFactory.loggerFactory.getClass() + ").");
				} else {
					LoggerFactory.loggerFactory = new StandardLoggerFactory();
					System.err.println("Logging erfolgt mit StandardLogger (Factory="
							+ LoggerFactory.loggerFactory.getClass() + ").");
				}
			}
		}
	}

	private static LoggerFactory createLoggerFactory(final String className) {
		LoggerFactory loggerFactory = null;
		try {
			final Class c = Class.forName(className);
			try {
				loggerFactory = (LoggerFactory) c.newInstance();
			} catch (final IllegalAccessException ex1) {
				ex1.printStackTrace();
			} catch (final InstantiationException ex1) {
				ex1.printStackTrace();
			}
		} catch (final ClassNotFoundException ex) {
		}
		return loggerFactory;
	}

	/**
	 * gibt die konkrete LoggerFactory zur&uuml;ck.
	 *
	 * @return konkrete LoggerFactory
	 */
	public static LoggerFactory getLoggerFactory() {
		return LoggerFactory.loggerFactory;
	}

	/**
	 * gibt einen Logger zur&uuml;ck, dessen Name gleich der aufrufenen Klasse
	 * ist (Bestimmmung &uuml;ber StackTrace).
	 *
	 * @return Logger
	 */
	public abstract Logger getLogger();

	/**
	 * gibt einen Logger mit dem Klassennamen der &uuml;bergebenen Klasse
	 * zur&uuml;ck.
	 *
	 * @param clazz
	 *            die Klasse für die bestimmung des Loggers
	 * @return Logger
	 */
	public abstract Logger getLogger(Class clazz);

	/**
	 * gibt einen Logger mit dem angebenen Namen zur&uuml;ck.
	 *
	 * @param name
	 *            der Name des gesuchten Loggers
	 * @return Logger
	 */
	public abstract Logger getLogger(String name);

	public abstract void setRootLevel(Logger.Level level);

	public abstract Object getRootLogger();

}
