/**
 * Segment 4 Datenübernahme und Aufbereitung (DUA), SWE 4.10 Ergänzung BASt-Band
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.bastband.gui;

import java.awt.Dimension;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collection;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 * Baum im JTreeView, der die Zaehlstellen und Konfigurationsbereiche darstellt
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class ZaehlStellenBaum extends JPanel {

	/**
	 * Autogenerated
	 */
	private static final long serialVersionUID = -195316447644079170L;
	/**
	 * Der Baum
	 */
	private final JTree baum;
	/**
	 * Die Flaeche auf die der Baum liegt
	 */
	private final JScrollPane baumView;
	/**
	 * Der Hauptknoten
	 */
	private DefaultMutableTreeNode hauptKnoten;

	/**
	 * Standardkonstruktor
	 */
	public ZaehlStellenBaum() {

		hauptKnoten = new KontrollKaestchenKnoten("Alle Konfigurationsbereiche");
		hauptKnoten.setAllowsChildren(true);
		baum = new JTree(hauptKnoten);
		((DefaultTreeModel) baum.getModel()).setAsksAllowsChildren(true);

		baum.setCellRenderer(new KontrollKaestchenKnotenRenderer());
		baum.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

		baum.putClientProperty("JTree.lineStyle", "Angled");
		baum.addMouseListener(new NodeSelectionListener(baum));

		// Create the scroll pane and add the tree to it.
		baumView = new JScrollPane(baum);

		// Add the split pane to this panel.
		add(baumView);

	}

	/**
	 * Stellt fest die Vorziehene Groesse des Baum - Widgets
	 */
	@Override
	public void setPreferredSize(final Dimension dim) {
		super.setPreferredSize(dim);
		baumView.setPreferredSize(dim);
	}

	/**
	 * Expandiert eine Zeile des Baums
	 *
	 * @param n
	 *            Riehenfolge der Zeile
	 */
	public void expandRow(final int n) {
		baum.expandRow(n);
	}

	/**
	 * Stellt fest einen neuen Hauptknoten ein
	 *
	 * @param hauptKnoten
	 *            neue Hauptknoten
	 */
	public void setHauptKnoten(final DefaultMutableTreeNode hauptKnoten) {
		this.hauptKnoten = hauptKnoten;
		final DefaultTreeModel model = (DefaultTreeModel) baum.getModel();
		model.setRoot(hauptKnoten);
		for (int i = 0; i < hauptKnoten.getChildCount(); i++) {
			if (hauptKnoten.getChildAt(i).getChildCount() == 0) {
				baum.expandRow(i + 1);
			}
		}
	}

	/**
	 * Aktualisiert die Menge der ausgewaehlten LZZ
	 *
	 * @param c
	 *            Menge von ausgewaehlten LZZ
	 */
	public void updateSelections(final Collection<String> c) {
		if (hauptKnoten instanceof KontrollKaestchenKnoten) {
			((KontrollKaestchenKnoten) hauptKnoten).updateSelections(c);
		}
	}

}

/**
 * Aktion beim Mausclick - Kaestchen unter der Maus auswaehlen
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
class NodeSelectionListener extends MouseAdapter {
	/**
	 * Der Baum
	 */
	JTree tree;

	/**
	 * Konstruktor
	 *
	 * @param tree
	 *            Der Baum
	 */
	NodeSelectionListener(final JTree tree) {
		this.tree = tree;
	}

	/**
	 * Wird beim Mausclick gerufen
	 *
	 */
	@Override
	public void mousePressed(final MouseEvent e) {
		final int x = e.getX();
		final int y = e.getY();
		final int row = tree.getRowForLocation(x, y);
		final TreePath path = tree.getPathForRow(row);
		// TreePath path = tree.getSelectionPath();
		if (path != null) {
			final KontrollKaestchenKnoten knoten = (KontrollKaestchenKnoten) path.getLastPathComponent();

			final boolean ausgewaehlt = !(knoten.isAusgewaehlt());
			knoten.setIstAusgewaehlt(ausgewaehlt);

			tree.repaint();
			((DefaultTreeModel) tree.getModel()).nodeChanged(knoten);

			// Fuer Wurzelknoten muss man uebermalen, sontst geht es nicht
			if (row == 0) {
				tree.revalidate();
				tree.repaint();
			}
		}
		super.mousePressed(e);
	}
}
